#ifndef EmergencyWatch_h
#define EmergencyWatch_h

#define THERMAL_PROTECTION_PIN 1153
#define GENERAL_EMERGENCY_PIN 1154 // Axis limit, indicates emergency when low
#define HAS_PRESSURE_PIN 1155
#define X_AXIS_NEGATIVE_LIMIT_PIN 139
#define X_AXIS_POSITIVE_LIMIT_PIN 140
#define Y_AXIS_NEGATIVE_LIMIT_PIN 141
#define Y_AXIS_POSITIVE_LIMIT_PIN 142
#define Z_AXIS_NEGATIVE_LIMIT_PIN 143
#define Z_AXIS_POSITIVE_LIMIT_PIN 1152

#define EMERGENCY_STATE_VAR 183

#define WATCHDOG_DELAY 5.0 // seconds after host stops to Trigger

#define THERMAL_PROTECTION_MESSAGE "Alerta de sobrecorrente dos motores está ativo"
#define GENERAL_EMERGENCY_MESSAGE "Alerta de emergencia geral está ativo"
#define HAS_PRESSURE_MESSAGE "Alerta de pressão baixa está ativo"
#define X_AXIS_NEGATIVE_LIMIT_MESSAGE "Eixo X atingiu o limite negativo"
#define X_AXIS_POSITIVE_LIMIT_MESSAGE "Eixo X atingiu o limite positivo"
#define Y_AXIS_NEGATIVE_LIMIT_MESSAGE "Eixo Y atingiu o limite negativo"
#define Y_AXIS_POSITIVE_LIMIT_MESSAGE "Eixo Y atingiu o limite positivo"
#define Z_AXIS_NEGATIVE_LIMIT_MESSAGE "Eixo Z atingiu o limite negativo"
#define Z_AXIS_POSITIVE_LIMIT_MESSAGE "Eixo Z atingiu o limite positivo"

// Monitor emergency safety variables
// If one or more of them are true, signalizes and set the emergency state
// If none of them are true, resets the emergency state
void InitialEmergencyMonitoring();

// Monitor emergency safety variables
// If one or more of them are true, signalizes and set the emergency state
// If none of them are true, resets the emergency state
void LoopEmergencyMonitoring();

// Receives a number of input to verify and a message to be shown if the input is true.
int WatchInputLowLogic(int input, char *message);

// Receives a number of input to verify and a message to be shown if the input is true.
int WatchInputHighLogic(int input, char *message);

void ServiceWatchdogStatus(void);
void WatchdogTripped(void);
void WatchdogOK(void);

// Sign that the emergency is raised to serve as a condition for other programs.
// Ex. not execute Init before clear emergency
void SetEmergencyState();

// Reset the emergency raised flag.
void ResetEmergencyState();

// Get a value that indicates wheter the emergency state is active or not.
// Returns 1 if it was executed
int GetEmergencyState();

#endif /* EmergencyWatch_h */
