#include "KMotionDef.h"
#include "ManualToolExchangeWatch.h"
#include "MillChangerDefs.h"
#include "PersistTool.h"
#include "Homing.h"

// Posible states for the tool exchange cycle
enum toolExchangeStates
{
    InitialToolExchangeState,
    GripperOpenState,
    ExtractorOnState
};

// Routine that monitors the inputs related to the manual tool exchange and runs the machine state
// that opens the gripper and the extractor
void ManualToolExchangeWatch(void)
{
    static enum toolExchangeStates states = InitialToolExchangeState; // Initializes the state machine
    static int blast=0,blastsolid=-1,bcount=0; // initialize the button state variables for switch debouncing
    static float value;
    static int Answer, tool;

    static double T0=0.0;  // remember the last time we turned on
    double T=Time_sec(); // get current Time_sec

    // Only monitors the next methods if no G Code or Homing is running
    // If not, clear the states machine and low the outputs
    if ((JOB_ACTIVE == 1) || (GetIsExecutingHoming() == 1)
        || (GetSpindleStatus() != 0) || (GetDrillBoxStatus() != 0) || (GetSawStatus() != 0))
    {
        //ClearBit(EXTRACT_OUTPUT); // commented because it generates a race condition while M6 command was executing
        //ClearBit(OPEN_TOOL_GRIPPER_OUTPUT);
        states = InitialToolExchangeState;
        return;
    }

    Debounce(ReadBit(EXCHANGE_TOOL_BUTTON_INPUT),&bcount,&blast,&blastsolid);
    int inputButtonState = blast;

    switch (states)
    {
        case InitialToolExchangeState:
            if (inputButtonState == 1)
            {
                SetBit(OPEN_TOOL_GRIPPER_OUTPUT);
                T0=T;  // set start time of cycle
                states = GripperOpenState;
                printf("Going to GripperOpenState. ManualToolExchangeWatch.\n");
            }
            // Keep InitialToolExchangeState, all outputs disabled
            break;
        case GripperOpenState:
            if (inputButtonState == 1)
            {
                if (T > T0 + GRIPPER_ON_STATE_TIME)  // are we out of the TIME_ON section of the cycle?
                {
                    T0 = 0.0;
                    SetBit(EXTRACT_OUTPUT);
                    states = ExtractorOnState;
                    printf("Going to ExtractorOnState. ManualToolExchangeWatch.\n");
                }
            }
            // Go back to InitialToolExchangeState, switch off outputs
            else
            {
                T0 = 0.0;
                ClearBit(OPEN_TOOL_GRIPPER_OUTPUT);
                states = InitialToolExchangeState;
                printf("Going to InitialToolExchangeState. ManualToolExchangeWatch.\n");
            }
            break;

        case ExtractorOnState:
            if (inputButtonState == 0)
            {
                ClearBit(EXTRACT_OUTPUT);
                ClearBit(OPEN_TOOL_GRIPPER_OUTPUT);
                states = InitialToolExchangeState;
                printf("Going to InitialToolExchangeState. ManualToolExchangeWatch.\n");

                Answer = InputBox("Tool in Spindle or -1",&value);
                tool = value;
                if (Answer)
                {
                    printf("Operator Canceled\n");
                }
                else if (!MillToolNumberValid(tool))  // check if invalid
                {
                    char s[80];
                    sprintf(s,"Numero %d da ferramenta atual é invalido\n",tool);
                    printf(s);
                    MsgBox(s, MB_ICONHAND | MB_OK);
                }
                else
                {
                    SaveCurrentMillTool(tool);
                    UpdateCurrentToolLabel(tool);
                    printf("Operator Entered Value of %d\n",tool);
                }
            }
            break;

        default:
            ClearBit(EXTRACT_OUTPUT);
            ClearBit(OPEN_TOOL_GRIPPER_OUTPUT);
            states = InitialToolExchangeState;
            break;
    }
}
