#include "KMotionDef.h"
#include "ProgramExecute.h"
#include "EmergencyWatch.h"

#ifndef TMP
#define TMP 10 // which spare persist to use to transfer data
#include "KflopToKMotionCNCFunctions.c"
#endif

void EndProgram()
{
    // Reset executing state from table 1, 2 and both.
    // Required to to the machine state execution from TableSelectionWatch.c
    if (persist.UserData[TABLE1_EXECUTING_VAR] == 1)
    {
        persist.UserData[TABLE1_EXECUTING_VAR] = 0;
    }

    if (persist.UserData[TABLE2_EXECUTING_VAR] == 1)
    {
        persist.UserData[TABLE2_EXECUTING_VAR] = 0;
    }

    if (persist.UserData[TABLE12_EXECUTING_VAR] == 1)
    {
        persist.UserData[TABLE12_EXECUTING_VAR] = 0;
    }

    // For safety reasons, turn off spindle and drills
    if (GetSpindleStatus() != SPINDLE_OFF)
    {
        StopSpindle();
    }

    // Turn off drill box if it is on.
    if (GetDrillBoxStatus() == 1)
    {
        ClearDrillOutputs();
    }

    if (GetSawStatus() > 0)
    {
        ClearSawOutputs();
    }

    if (ReadBit(MAGAZINE_CLOSED_INPUT))
    {
        ClearBit(CLOSE_MAGAZINE_OUTPUT);
    }
}

void HaltAndWarn(char *message)
{
    DoPC(PC_COMM_HALT);
    EndProgram();
    Delay_sec(1);
    MsgBoxNoWait(message, MB_ICONEXCLAMATION);
    Delay_sec(1);
    DoPC(PC_COMM_RESTART);
    Delay_sec(1);
}

int IsProgramAbleToExecute()
{
    if (GetEmergencyState())
    {
        HaltAndWarn("Resolva a condicao de emergencia antes de executar o programa");
        return 0;
    }

    if (GetInitExecuted() == 0 || GetHomingExecuted() == 0 || GetIsExecutingHoming() == 1)
    {
        HaltAndWarn("Execute a inicializacao e zeramento antes de executar o programa");
        return 0;
    }
    
    if (ReadBit(MAGAZINE_OPENED_INPUT))
    {
        HaltAndWarn("Feche o magazine de ferramentas antes de executar o programa");
        return 0;
    }
    
    // If everything is OK returns 1 (True), thus the program is able to execute.
    return 1;
}
