//#include "Homing.c"
//#include "InitConfig.c"
#include "MovementDefs.h"
#include "Magazine.h"
#include "TableSelectionWatch.h"
#include "ProgramExecute.h"
#include "KMotionDef.h"

#ifndef TMP
#define TMP 10 // which spare persist to use to transfer data
#include "KflopToKMotionCNCFunctions.c"
#endif

// Offset Auxiliar function declarations
int IsOffsetChecked();
void ApplyOffsetToTableFixture(int tableSelected);
void GetFixtureValuesFromTable(int table, double *OriginOffsetX, double *OriginOffsetY, double *OriginOffsetZ);
void ApplyOffsetToAxisFromFixture(double originValue, int axis, int offsetVar, int fixture);
void TransferAxisFixtureValue(double value, int axis, int fixture);

// Status label function declaration
void UpdateStatusLabel1(char *message);
void UpdateStatusLabel2(char *message);
void UpdateStatusLabel12(char *message);
char *TableStateToString(int state);

// Posible states for the table selection
enum tableSelectionStates
{
    InitialState,
    TableSelected,
    ReadyToExecute,
    SafeZBegin,
    ProgramExecution,
    SafeZEnd
};

#define UNAVAILABLE_STATE "Indisponivel"

int tableExecuting = 0;

void ClearTableOutputs()
{
    // Table 1
    ClearBit(AUXILIARY_POSITIONING1_OUTPUT);
    ClearBit(VACUUM1_OUTPUT);
    // Table 2
    ClearBit(AUXILIARY_POSITIONING2_OUTPUT);
    ClearBit(AUXILIARY_POSITIONING_MIDDLE_OUTPUT);
    ClearBit(VACUUM2_OUTPUT);
}

// Move Axis X at specified Speed
// return 0 = success, 1 if axis disabled
int MoveX(float x, float Speed)
{
	MoveAtVel(X_AXIS, x / FACTOR_X, Speed / FACTOR_X);
	
	return 0;  //success
}

// Move Axis Z at specified Speed
// return 0 = success, 1 if axis disabled
int MoveZ(float z, float Speed)
{
	MoveAtVel(Z_AXIS, z / FACTOR_Z, Speed / FACTOR_Z);
	
	return 0;  //success
}

Table1SelectionWatch()
{
    static enum tableSelectionStates states = InitialState; // Initializes the state machine
    static int tableButtonChangeState=-1,tlast=0,tlastsolid=-1,tcount=0; // initialize the table selection button state variables for switch debouncing
    static int vacuumButtonChangeState=-1,vlast=0,vlastsolid=-1,vcount=0; // initialize the vacuum button state variables for switch debouncing

    // To reset outputs when an init is performed
    static int firstTime = 1;
    if (firstTime == 1)
    {
        ClearBit(AUXILIARY_POSITIONING1_OUTPUT);
        ClearBit(VACUUM1_OUTPUT);
        firstTime = 0;
    }

    tableExecuting = persist.UserData[TABLE_EXECUTING_VAR];

    tableButtonChangeState = Debounce(ReadBit(TABLE1_BUTTON_INPUT),&tcount,&tlast,&tlastsolid);
    vacuumButtonChangeState = Debounce(ReadBit(VACUUM1_BUTTON_INPUT),&vcount,&vlast,&vlastsolid);

    switch (states)
    {
    case InitialState:
        if (tableButtonChangeState == 1 && tableExecuting != TABLE1_AND_2)
        {
            SetBit(AUXILIARY_POSITIONING1_OUTPUT);
            states = TableSelected;
            printf("Going to Table1Selected. TableSelectionWatch.\n");
            char *message = TableStateToString(TableSelected);
            UpdateStatusLabel1(message);
        }

        break;
    
    case TableSelected:
        if (vacuumButtonChangeState == 1)
        {
            if (ReadBit(VACUUM1_OUTPUT) == 0)
            {
                SetBit(VACUUM1_OUTPUT);
            }
            else
            {
                ClearBit(VACUUM1_OUTPUT);
            }
        }

        if(tableButtonChangeState == 1 && ReadBit(VACUUM1_CONFIRMATION_INPUT))
        {
            ClearBit(AUXILIARY_POSITIONING1_OUTPUT);
            states = ReadyToExecute;
            printf("Going to ReadyToExecute1. TableSelectionWatch.\n");
            char *message = TableStateToString(ReadyToExecute);
            UpdateStatusLabel1(message);
        }
        else if (tableButtonChangeState == 1)
        {
            ClearBit(AUXILIARY_POSITIONING1_OUTPUT);
            states = InitialState;
            printf("Going to InitialState1. TableSelectionWatch.\n");
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel1(message);
        }
        
        break;
    
    case ReadyToExecute:
        if (!JOB_ACTIVE && tableExecuting == 0)
        {
            persist.UserData[TABLE_EXECUTING_VAR] = TABLE1;
            persist.UserData[TABLE1_EXECUTING_VAR] = 1;
            MoveZ(0, 10);
            states = SafeZBegin;
            printf("Going to SafeZBegin1. TableSelectionWatch.\n");
            char *message = TableStateToString(SafeZBegin);
            UpdateStatusLabel1(message);
            UpdateStatusLabel12(UNAVAILABLE_STATE);
        }

        break;

    case SafeZBegin:
        if(CheckDone(Z_AXIS) == 0) break;
        if (IsProgramAbleToExecute() == 0) 
        {
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            states = InitialState;
            printf("Going to InitialState1. TableSelectionWatch.\n");
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel1(message);
            UpdateStatusLabel12(message);
            break;
        }
        if (IsOffsetChecked())
        {
            printf("Offset checked. TableSelectionWatch.\n");
            ApplyOffsetToTableFixture(TABLE1);
            MDI("G56");
        }
        else 
        {
            MDI("G54");
        }

        DoPC(PC_COMM_EXECUTE);
        states = ProgramExecution;
        printf("Going to ProgramExecution1. TableSelectionWatch.\n");
        char *message = TableStateToString(ProgramExecution);
        UpdateStatusLabel1(message);

        break;

    case ProgramExecution:
        if (!JOB_ACTIVE && persist.UserData[TABLE1_EXECUTING_VAR] == 0)
        {
            MoveZ(0, 10);
            states = SafeZEnd;
            printf("Going to SafeZEnd1. TableSelectionWatch.\n");
            char *message = TableStateToString(SafeZEnd);
            UpdateStatusLabel1(message);
        }

        if (!ReadBit(VACUUM1_CONFIRMATION_INPUT))
        {
            DoPC(PC_COMM_HALT);
            ClearBit(VACUUM1_OUTPUT);
            states = InitialState;
            printf("Going to InitialState1. TableSelectionWatch.\n");
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            EndProgram();
            Delay_sec(0.6);
            DoPC(PC_COMM_RESTART);
            Delay_sec(0.6);
            MsgBoxNoWait("Perda de confirmacao de vacuo! Parada de programa ativada.", MB_ICONEXCLAMATION);
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel1(message);
            UpdateStatusLabel12(message);
        }

        break;

    case SafeZEnd:
        if(CheckDone(Z_AXIS))
        {
            ClearBit(VACUUM1_OUTPUT);
            states = InitialState;
            printf("Going to InitialState1. TableSelectionWatch.\n");
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel1(message);
            UpdateStatusLabel12(message);
        }
        break;

    default:
        states = InitialState;
        break;
    }
}

Table2SelectionWatch()
{
    static enum tableSelectionStates states = InitialState; // Initializes the state machine
    static int tableButtonChangeState=-1,tlast=0,tlastsolid=-1,tcount=0; // initialize the table selection button state variables for switch debouncing
    static int vacuumButtonChangeState=-1,vlast=0,vlastsolid=-1,vcount=0; // initialize the vacuum button state variables for switch debouncing

    // To reset outputs when an init is performed
    static int firstTime = 1;
    if (firstTime == 1)
    {
        ClearBit(AUXILIARY_POSITIONING2_OUTPUT);
        ClearBit(AUXILIARY_POSITIONING_MIDDLE_OUTPUT);
        ClearBit(VACUUM2_OUTPUT);
        firstTime = 0;
    }

    tableExecuting = persist.UserData[TABLE_EXECUTING_VAR];

    tableButtonChangeState = Debounce(ReadBit(TABLE2_BUTTON_INPUT),&tcount,&tlast,&tlastsolid);
    vacuumButtonChangeState = Debounce(ReadBit(VACUUM2_BUTTON_INPUT),&vcount,&vlast,&vlastsolid);

    switch (states)
    {
    case InitialState:
        if (tableButtonChangeState == 1 && tableExecuting != TABLE1_AND_2)
        {
            SetBit(AUXILIARY_POSITIONING2_OUTPUT);
            SetBit(AUXILIARY_POSITIONING_MIDDLE_OUTPUT);
            states = TableSelected;
            printf("Going to Table2Selected. TableSelectionWatch.\n");
            char *message = TableStateToString(TableSelected);
            UpdateStatusLabel2(message);
        }

        break;
    
    case TableSelected:
        if (vacuumButtonChangeState == 1)
        {
            if (ReadBit(VACUUM2_OUTPUT) == 0)
            {
                SetBit(VACUUM2_OUTPUT);
            }
            else
            {
                ClearBit(VACUUM2_OUTPUT);
            }
        }

        if(tableButtonChangeState == 1 && ReadBit(VACUUM2_CONFIRMATION_INPUT))
        {
            ClearBit(AUXILIARY_POSITIONING2_OUTPUT);
            ClearBit(AUXILIARY_POSITIONING_MIDDLE_OUTPUT);
            states = ReadyToExecute;
            printf("Going to ReadyToExecute2. TableSelectionWatch.\n");
            char *message = TableStateToString(ReadyToExecute);
            UpdateStatusLabel2(message);
        }
        else if (tableButtonChangeState == 1)
        {
            ClearBit(AUXILIARY_POSITIONING2_OUTPUT);
            ClearBit(AUXILIARY_POSITIONING_MIDDLE_OUTPUT);
            states = InitialState;
            printf("Going to InitialState2. TableSelectionWatch.\n");
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel2(message);
        }
        
        break;
    
    case ReadyToExecute:
        if (!JOB_ACTIVE && tableExecuting == 0)
        {
            persist.UserData[TABLE_EXECUTING_VAR] = TABLE2;
            persist.UserData[TABLE2_EXECUTING_VAR] = 1;
            MoveZ(0, 10);
            states = SafeZBegin;
            printf("Going to SafeZBegin2. TableSelectionWatch.\n");
            char *message = TableStateToString(SafeZBegin);
            UpdateStatusLabel2(message);
            UpdateStatusLabel12(UNAVAILABLE_STATE);
        }

        break;

    case SafeZBegin:
        if(CheckDone(Z_AXIS) == 0) break;
        if (IsProgramAbleToExecute() == 0) 
        {
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            states = InitialState;
            printf("Going to InitialState2. TableSelectionWatch.\n");
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel2(message);
            UpdateStatusLabel12(message);
            break;
        }
        if (IsOffsetChecked())
        {
            printf("Offset checked. TableSelectionWatch.\n");
            ApplyOffsetToTableFixture(TABLE2);
            MDI("G57");
        }
        else 
        {
            MDI("G55");
        }

        DoPC(PC_COMM_EXECUTE);
        states = ProgramExecution;
        printf("Going to ProgramExecution2. TableSelectionWatch.\n");
        char *message = TableStateToString(ProgramExecution);
        UpdateStatusLabel2(message);

        break;

    case ProgramExecution:
        if (!JOB_ACTIVE && persist.UserData[TABLE2_EXECUTING_VAR] == 0)
        {
            MoveZ(0, 10);
            states = SafeZEnd;
            printf("Going to SafeZEnd2. TableSelectionWatch.\n");
            char *message = TableStateToString(SafeZEnd);
            UpdateStatusLabel2(message);
        }

        if (!ReadBit(VACUUM2_CONFIRMATION_INPUT))
        {
            DoPC(PC_COMM_HALT);
            ClearBit(VACUUM2_OUTPUT);
            states = InitialState;
            printf("Going to InitialState2. TableSelectionWatch.\n");
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            EndProgram();
            Delay_sec(0.6);
            DoPC(PC_COMM_RESTART);
            Delay_sec(0.6);
            MsgBoxNoWait("Perda de confirmacao de vacuo! Parada de programa ativada.", MB_ICONEXCLAMATION);
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel2(message);
            UpdateStatusLabel12(message);
        }

        break;

    case SafeZEnd:
        if(CheckDone(Z_AXIS))
        {
            ClearBit(VACUUM2_OUTPUT);
            states = InitialState;
            printf("Going to InitialState2. TableSelectionWatch.\n");
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel2(message);
            UpdateStatusLabel12(message);
        }
        break;

    default:
        states = InitialState;
        break;
    }
}

Table1And2SelectionWatch()
{
    static enum tableSelectionStates states = InitialState; // Initializes the state machine
    static int tableButtonChangeState1=-1,tlast1=0,tlastsolid1=-1,tcount1=0; // initialize the table selection button state variables for switch debouncing
    static int tableButtonChangeState2=-1,tlast2=0,tlastsolid2=-1,tcount2=0; // initialize the table selection button state variables for switch debouncing
    static int vacuumButtonChangeState1=-1,vlast1=0,vlastsolid1=-1,vcount1=0; // initialize the vacuum button state variables for switch debouncing
    static int vacuumButtonChangeState2=-1,vlast2=0,vlastsolid2=-1,vcount2=0; // initialize the vacuum button state variables for switch debouncing

    tableExecuting = persist.UserData[TABLE_EXECUTING_VAR];

    tableButtonChangeState1 = Debounce(ReadBit(TABLE12_BUTTON_INPUT_1),&tcount1,&tlast1,&tlastsolid1);
    tableButtonChangeState2 = Debounce(ReadBit(TABLE12_BUTTON_INPUT_2),&tcount2,&tlast2,&tlastsolid2);
    vacuumButtonChangeState1 = Debounce(ReadBit(VACUUM12_BUTTON_INPUT_1),&vcount1,&vlast1,&vlastsolid1);
    vacuumButtonChangeState2 = Debounce(ReadBit(VACUUM12_BUTTON_INPUT_2),&vcount2,&vlast2,&vlastsolid2);

    switch (states)
    {
    case InitialState:
        if ((tableButtonChangeState1 == 1 || tableButtonChangeState2 == 1) && 
            (tableExecuting != TABLE1 && tableExecuting != TABLE2))
        {
            SetBit(AUXILIARY_POSITIONING1_OUTPUT);
            SetBit(AUXILIARY_POSITIONING2_OUTPUT);
            states = TableSelected;
            printf("Going to Table12Selected. TableSelectionWatch.\n");
            char *message = TableStateToString(TableSelected);
            UpdateStatusLabel12(message);
        }

        break;
    
    case TableSelected:
        if (vacuumButtonChangeState1 == 1 || vacuumButtonChangeState2 == 1)
        {
            if (ReadBit(VACUUM1_OUTPUT) == 0 || ReadBit(VACUUM2_OUTPUT) == 0)
            {
                SetBit(VACUUM1_OUTPUT);
                SetBit(VACUUM2_OUTPUT);
            }
            else
            {
                ClearBit(VACUUM1_OUTPUT);
                ClearBit(VACUUM2_OUTPUT);
            }
        }

        if((tableButtonChangeState1 == 1 || tableButtonChangeState2 == 1) && ReadBit(VACUUM1_CONFIRMATION_INPUT) && ReadBit(VACUUM2_CONFIRMATION_INPUT) && tableExecuting == 0)
        {
            ClearBit(AUXILIARY_POSITIONING1_OUTPUT);
            ClearBit(AUXILIARY_POSITIONING2_OUTPUT);
            states = ReadyToExecute;
            printf("Going to ReadyToExecute12. TableSelectionWatch.\n");
            char *message = TableStateToString(ReadyToExecute);
            UpdateStatusLabel12(message);
        }
        else if (tableButtonChangeState1 == 1 || tableButtonChangeState2 == 1)
        {
            ClearBit(AUXILIARY_POSITIONING1_OUTPUT);
            ClearBit(AUXILIARY_POSITIONING2_OUTPUT);
            states = InitialState;
            printf("Going to InitialState12. TableSelectionWatch.\n");
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel12(message);
        }
        
        break;
    
    case ReadyToExecute:
        if (!JOB_ACTIVE && tableExecuting == 0)
        {
            persist.UserData[TABLE_EXECUTING_VAR] = TABLE1_AND_2;
            persist.UserData[TABLE12_EXECUTING_VAR] = 1;
            MoveZ(0, 10);
            states = SafeZBegin;
            printf("Going to SafeZBegin12. TableSelectionWatch.\n");
            char *message = TableStateToString(SafeZBegin);
            UpdateStatusLabel12(message);
            UpdateStatusLabel1(UNAVAILABLE_STATE);
            UpdateStatusLabel2(UNAVAILABLE_STATE);
        }

        break;

    case SafeZBegin:
        if(CheckDone(Z_AXIS) == 0) break;
        if (IsProgramAbleToExecute() == 0) 
        {
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            states = InitialState;
            printf("Going to InitialState12. TableSelectionWatch.\n");
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel12(message);
            UpdateStatusLabel1(message);
            UpdateStatusLabel2(message);
            break;
        }
        if (IsOffsetChecked())
        {
            printf("Offset checked. TableSelectionWatch.\n");
            ApplyOffsetToTableFixture(TABLE1);
            MDI("G56");
        }
        else 
        {
            MDI("G54");
        }

        DoPC(PC_COMM_EXECUTE);
        states = ProgramExecution;
        printf("Going to ProgramExecution12. TableSelectionWatch.\n");
        char *message = TableStateToString(ProgramExecution);
        UpdateStatusLabel12(message);

        break;

    case ProgramExecution:
        if (!JOB_ACTIVE && persist.UserData[TABLE12_EXECUTING_VAR] == 0)
        {
            MoveZ(0, 10);
            states = SafeZEnd;
            printf("Going to SafeZEnd12. TableSelectionWatch.\n");
            char *message = TableStateToString(SafeZEnd);
            UpdateStatusLabel12(message);
        }

        if (!(ReadBit(VACUUM1_CONFIRMATION_INPUT) && ReadBit(VACUUM2_CONFIRMATION_INPUT)))
        {
            DoPC(PC_COMM_HALT);
            ClearBit(VACUUM1_OUTPUT);
            ClearBit(VACUUM2_OUTPUT);
            states = InitialState;
            printf("Going to InitialState12. TableSelectionWatch.\n");
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            EndProgram();
            Delay_sec(0.6);
            DoPC(PC_COMM_RESTART);
            Delay_sec(0.6);
            MsgBoxNoWait("Perda de confirmacao de vacuo! Parada de programa ativada.", MB_ICONEXCLAMATION);
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel12(message);
            UpdateStatusLabel1(message);
            UpdateStatusLabel2(message);
        }

        break;

    case SafeZEnd:
        if(CheckDone(Z_AXIS))
        {
            ClearBit(VACUUM1_OUTPUT);
            ClearBit(VACUUM2_OUTPUT);
            states = InitialState;
            printf("Going to InitialState12. TableSelectionWatch.\n");
            persist.UserData[TABLE_EXECUTING_VAR] = 0;
            char *message = TableStateToString(InitialState);
            UpdateStatusLabel12(message);
            UpdateStatusLabel1(message);
            UpdateStatusLabel2(message);
        }
        break;

    default:
        states = InitialState;
        break;
    }
}

int IsOffsetChecked()
{
    return (ReadBit(X_OFFSET_CHECK_VIRTUAL_BIT) ||
            ReadBit(Y_OFFSET_CHECK_VIRTUAL_BIT) ||
            ReadBit(Z_OFFSET_CHECK_VIRTUAL_BIT));
}

void ApplyOffsetToTableFixture(int tableSelected)
{
    double OriginOffsetX, OriginOffsetY, OriginOffsetZ;
    int fixture;
    GetFixtureValuesFromTable(tableSelected, &OriginOffsetX, &OriginOffsetY, &OriginOffsetZ);
    printf("Fixture values: X - %f, Y - %f, Z - %f. TableSelectionWatch.\n", OriginOffsetX, OriginOffsetY, OriginOffsetZ);
    if (tableSelected == TABLE1 || tableSelected == TABLE1_AND_2)
    {
        fixture = 3;
    }
    else
    {
        fixture = 4;
    }

    TransferAxisFixtureValue(OriginOffsetX, X_AXIS, fixture);
    TransferAxisFixtureValue(OriginOffsetY, Y_AXIS, fixture);
    TransferAxisFixtureValue(OriginOffsetZ, Z_AXIS, fixture);

    if (ReadBit(X_OFFSET_CHECK_VIRTUAL_BIT))
    {
        printf("Apply offset to X. TableSelectionWatch.\n");
        ApplyOffsetToAxisFromFixture(OriginOffsetX, X_AXIS, X_OFFSET_VALUE_VAR, fixture);
    }

    if (ReadBit(Y_OFFSET_CHECK_VIRTUAL_BIT))
    {
        printf("Apply offset to Y. TableSelectionWatch.\n");
        ApplyOffsetToAxisFromFixture(OriginOffsetY, Y_AXIS, Y_OFFSET_VALUE_VAR, fixture);
    }

    if (ReadBit(Z_OFFSET_CHECK_VIRTUAL_BIT))
    {
        printf("Apply offset to Z. TableSelectionWatch.\n");
        ApplyOffsetToAxisFromFixture(OriginOffsetZ, Z_AXIS, Z_OFFSET_VALUE_VAR, fixture);
    }

    DoPC(PC_COMM_UPDATE_FIXTURE);
}

void GetFixtureValuesFromTable(int table, double *OriginOffsetX, double *OriginOffsetY, double *OriginOffsetZ)
{
    GetOriginOffset(OriginOffsetX, table, X_AXIS);
    GetOriginOffset(OriginOffsetY, table, Y_AXIS);
    GetOriginOffset(OriginOffsetZ, table, Z_AXIS);
}

void ApplyOffsetToAxisFromFixture(double originValue, int axis, int offsetVar, int fixture)
{
    double offset, newOrigin;
    if (GetEditControlDouble(&offset, offsetVar, 1000))
	{
		printf("Error Reading Edit Control Var=%d\n", offsetVar);
		return;
	}

    if ((offset < 0) && (axis == X_AXIS || axis == Z_AXIS))
    {
        MsgBoxNoWait("Offsets negativos nos eixos X e Z nao sao validos por seguranca", MB_ICONEXCLAMATION);
		return;
    }
    if ((offset > 0) && (axis == Y_AXIS))
    {
        MsgBoxNoWait("Offsets positivos no eixo Y nao sao validos por seguranca", MB_ICONEXCLAMATION);
		return;
    }

    newOrigin = originValue + offset;
    printf("New origin: %f, Old origin: %f, Offset: %f. TableSelectionWatch.\n", newOrigin, originValue, offset);
    SetUserDataDouble(TMP, newOrigin);
    SetVars(5201+fixture*20+axis, 1, TMP);
}

void TransferAxisFixtureValue(double value, int axis, int fixture)
{
    printf("Value: %f, axis: %d, fixure: %d. TableSelectionWatch.\n", value, axis, fixture);
    SetUserDataDouble(TMP, value);
    SetVars(5201+fixture*20+axis, 1, TMP);
}

void UpdateStatusLabel1(char *message)
{
    printf("UpdateStatusLabel\n");
    char s[80];
    // Now compute and form result
	sprintf(s,"%s: %s",TABLE_1_NAME, message);
    // Put it onto the Screen
	DROLabel(1000, TABLE_1_STATUS_LABEL_VAR, s);
}

void UpdateStatusLabel2(char *message)
{
    printf("UpdateStatusLabel\n");
    char s[80];
    // Now compute and form result
	sprintf(s,"%s: %s",TABLE_2_NAME, message);
    // Put it onto the Screen
	DROLabel(1010, TABLE_2_STATUS_LABEL_VAR, s);
}

void UpdateStatusLabel12(char *message)
{
    printf("UpdateStatusLabel\n");
    char s[80];
    // Now compute and form result
	sprintf(s,"%s: %s",TABLE_12_NAME, message);
    // Put it onto the Screen
	DROLabel(1020, TABLE_12_STATUS_LABEL_VAR, s);
}

char *TableStateToString(int state)
{
    switch (state)
    {
    case InitialState:
        return "Disponivel";

    case TableSelected:
        return "Selecionada";

    case ReadyToExecute:
        return "Pronta para execucao";

    case SafeZBegin:
        return "Deslocamento seguro em Z";

    case ProgramExecution:
        return "Programa em Execucao";

    case SafeZEnd:
        return "Deslocamento seguro em Z";
    
    default:
        return "Estado desconhecido";
    }
}
